@@ -4,7 +4,7 @@ use strict;
 # If you change the class path take a look in get_defaults too, please!
 package Munin::Common::Defaults;
 
-# $Id: Defaults.pm 2951 2009-11-10 20:13:54Z knuthaug $
+# $Id$
 
 use English qw(-no_match_vars);
 use File::Basename qw(dirname);
@@ -25,15 +25,17 @@ our $MUNIN_DOCDIR     = q{/usr/local/doc};
 our $MUNIN_LIBDIR     = q{/usr/local/libexec/munin};
 our $MUNIN_HTMLDIR    = q{/var/www/htdocs/munin};
 our $MUNIN_CGIDIR     = q{/var/www/cgi-bin};
+our $MUNIN_CGITMPDIR     = '';
 our $MUNIN_DBDIR      = q{/var/db/munin};
 our $MUNIN_PLUGSTATE  = q{/var/db/munin-pluginstate};
+our $MUNIN_SPOOLDIR   = q{/var/db/munin/spool};
 our $MUNIN_MANDIR     = q{/usr/local/man};
 our $MUNIN_LOGDIR     = q{/var/log/munin};
 our $MUNIN_STATEDIR   = q{/var/run/munin};
 our $MUNIN_USER       = q{_munin};
 our $MUNIN_GROUP      = q{_munin};
 our $MUNIN_PLUGINUSER = q{nobody};
-our $MUNIN_VERSION    = q{1.4.7};
+our $MUNIN_VERSION    = q{2.0.10};
 our $MUNIN_PERL       = q{/usr/bin/perl};
 our $MUNIN_PERLLIB    = q{/usr/local/libdata/perl5/site_perl};
 our $MUNIN_GOODSH     = q{/bin/sh};
@@ -90,7 +92,7 @@ scripts.
 
 =head1 PACKAGE VARIABLES
 
-See L<http://munin.projects.linpro.no/wiki/MuninInstallProcedure> for
+See L<http://munin-monitoring.org/wiki/MuninInstallProcedure> for
 more information on the variables provided by this package.
 
 
