@@ -26,7 +26,7 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
-$Id: LimitsOld.pm 4400 2011-09-29 00:45:15Z bldewolf $
+$Id$
 
 =end comment
 
@@ -58,9 +58,11 @@ my $do_version     = 0;
 my @limit_hosts    = ();
 my @limit_services = ();
 my @limit_contacts = ();
+my @always_send    = ();
 my $stdout         = 0;
-my %notes          = ();
 my $force          = 0;
+my $force_run_as_root = 0;
+my %notes          = ();
 my $config;
 my $oldnotes;
 my $modified     = 0;
@@ -87,6 +89,8 @@ sub limits_startup {
         "debug!"    => \$DEBUG,
         "stdout!"   => \$stdout,
         "force!"    => \$force,
+        "always-send=s" => \@always_send,
+        "force-run-as-root!" => \$force_run_as_root,
         "version!"  => \$do_version,
         "help"      => \$do_usage
         );
@@ -94,9 +98,13 @@ sub limits_startup {
     print_usage_and_exit()   if $do_usage;
     print_version_and_exit() if $do_version;
 
-    exit_if_run_by_super_user();
+    exit_if_run_by_super_user() unless $force_run_as_root;
 
-    $config = &munin_config($conffile);
+    @always_send = qw{ok warning critical unknown} if $force;
+
+    munin_readconfig_base($conffile);
+    # XXX: check if it does actualy need that part
+    $config = munin_readconfig_part('datafile', 0);
 
     logger_open($config->{'logdir'});
     logger_debug() if $DEBUG;
@@ -115,7 +123,7 @@ sub limits_main {
 
     munin_runlock("$config->{rundir}/munin-limits.lock");
 
-    $oldnotes = &munin_readconfig($config->{'dbdir'} . "/limits", 1, 1);
+    $oldnotes = &munin_readconfig_part('limits', 1);
 
     initialize_for_nagios();
 
@@ -126,6 +134,7 @@ sub limits_main {
     close_pipes();
 
     &munin_writeconfig("$config->{dbdir}/limits", \%notes);
+    &munin_writeconfig_storable("$config->{dbdir}/limits.storable", \%notes);
 
     $update_time = sprintf("%.2f", (Time::HiRes::time - $update_time));
 
@@ -205,7 +214,15 @@ Options:
     --help		View this message.
     --debug		View debug messages.
     --stdout		Log to stdout as well as the log file.
-    --force		Send messages even if they shouldn't normally be sent.
+    --always-send <severity list>
+                        Send messages to contacts even if state has
+                        not changed since the last run. The list is a
+                        space or comma separated list of severities.
+                        Choose from one or more of \"critical\",
+                        \"warning\", \"unknown\" and \"ok\".
+    --force		Alias for \"--always-send ok,warning,critical,unknown\".
+                        Overrides --always-send command line, as well as the
+                        always_send contact configuration options.
     --service <service>	Limit notified services to <service>. Multiple 
     			--service options may be supplied.
     --host <host>	Limit notified hosts to <host>. Multiple --host 
@@ -264,8 +281,8 @@ sub get_full_group_path {
     my $parent     = munin_get_parent($group);
     my $name       = get_notify_name($group);
 
-    if (defined $parent and munin_get_node_name($parent) ne "#%#root") {
-	return (get_full_group_path($parent) . " :: " . $name);
+    if (defined $parent and munin_get_node_name($parent) ne "root") {
+	return (get_full_group_path($parent) . "-" . $name);
     } else {
 	return $name;
     }
@@ -274,6 +291,8 @@ sub get_full_group_path {
 sub process_service {
     my $hash       = shift || return;
     my $hobj       = get_host_node($hash);
+    my $host       = munin_get_node_name($hobj);
+    my $hostalias  = get_notify_name($hobj);
     my $service    = munin_get_node_name($hash);
     my $hparentobj = munin_get_parent($hobj);
     my $parent     = munin_get_node_name($hobj);
@@ -283,7 +302,7 @@ sub process_service {
     if (!ref $hash) {
 	LOGCROAK("I was passed a non-hash!");
     }
-
+    return if (@limit_hosts and !grep (/^$host$/, @limit_hosts));
     return if (@limit_services and !grep (/^$service$/, @limit_services));
 
     DEBUG "[DEBUG] processing service: $service";
@@ -292,35 +311,70 @@ sub process_service {
     $hash->{'fields'} = join(' ', map {munin_get_node_name($_)} @$children);
     $hash->{'plugin'} = $service;
     $hash->{'graph_title'} = get_full_service_name($hash);
-    $hash->{'host'}  = get_notify_name($hobj);
+    $hash->{'host'}  = $hostalias;
     $hash->{'group'} = get_full_group_path($hparentobj);
     $hash->{'worst'} = "ok";
     $hash->{'worstid'} = 0 unless defined $hash->{'worstid'};
 
+    my $state_file = sprintf ('%s/state-%s-%s.storable', $config->{dbdir}, $hash->{group}, $host);
+    DEBUG "[DEBUG] state_file: $state_file";
+    my $state = munin_read_storable($state_file) || {};
+
     foreach my $field (@$children) {
         next if (!defined $field or ref($field) ne "HASH");
         my $fname   = munin_get_node_name($field);
         my $fpath   = munin_get_node_loc($field);
         my $onfield = munin_get_node($oldnotes, $fpath);
+	my $oldstate= '';
 
 	# Test directly here as get_limits is in truth recursive and
 	# that fools us when processing multigraphs.
 	next if (!defined($field->{warning}) and !defined($field->{critical}));
 
+	# get the old state if there is one, or leave it empty.
+	if ( defined($onfield) or
+	     defined($onfield->{"state"}) ) {
+	    $oldstate = $onfield->{"state"};
+	}
+
         my ($warn, $crit, $unknown_limit) = get_limits($field);
 
         # Skip fields without warning/critical definitions
         next if (!defined $warn and !defined $crit);
 
         DEBUG "[DEBUG] processing field: " . join('::', @$fpath);
+        DEBUG "[DEBUG] field: " . munin_dumpconfig_as_str($field);
+	my $value;
+    	{
+		my $rrd_filename = munin_get_rrd_filename($field);
+		my ($current_updated_timestamp, $current_updated_value) = @{ $state->{value}{"$rrd_filename:42"}{current} || [ ] };
+		my ($previous_updated_timestamp, $previous_updated_value) = @{ $state->{value}{"$rrd_filename:42"}{previous} || [ ] };
 
-        my $filename = munin_get_rrd_filename($field);
-        my $value    = munin_fetch("$filename");
-
-	DEBUG "[DEBUG] rrd filename is: $filename";
+		my $heartbeat = 600; # XXX - $heartbeat is a fixed 10 min (2 runs of 5 min).
+		if (! $field->{type} || $field->{type} eq "GAUGE" || $field->{type} eq "ABSOLUTE") {
+			$value = $current_updated_value;
+		} elsif (! defined $current_updated_value || ! defined $previous_updated_value || $current_updated_timestamp == $previous_updated_timestamp) {
+			# No derive computing possible. Report unknown.
+			$value = "U";
+		} elsif (time > $current_updated_timestamp + $heartbeat) {
+			# Current value is too old. Report unknown. 
+			$value = "U";
+		} elsif ($current_updated_timestamp > $previous_updated_timestamp + $heartbeat) {
+			# Old value is too old. Report unknown. 
+			$value = "U";
+		} elsif ($field->{type} eq "COUNTER" && $current_updated_value < $previous_updated_value) {
+			# COUNTER never decrease. Report unknown.
+			$value = "U";
+		} elsif ($current_updated_value eq "U" || $previous_updated_value eq "U" ) {
+			# One of the values is unknown. Report unknown.
+			$value = "U";
+		} else {
+			$value = ($current_updated_value - $previous_updated_value) / ($current_updated_timestamp - $previous_updated_timestamp);
+		}
+	}
 
         # De-taint.
-        if (!defined $value) {
+        if (!defined $value || $value eq "U") {
             $value = "unknown";
         }
         else {
@@ -351,9 +405,7 @@ sub process_service {
                     : "Value is unknown.";
             my $num_unknowns;
 
-            if (   !defined $onfield
-                or !defined $onfield->{"state"}
-                or $onfield->{"state"} ne "unknown") {
+            if ( $oldstate ne "unknown") {
                 $hash->{'state_changed'} = 1;
             }
             else {
@@ -431,12 +483,8 @@ sub process_service {
                         . ") exceeded"
                 ));
 
-            if (   !defined $onfield
-                or !defined $onfield->{"state"}
-                or $onfield->{"state"} ne "critical") {
-
+            if ( $oldstate ne "critical") {
                 $hash->{'state_changed'} = 1;
-
             }
         }
         elsif ((defined($warn->[0]) and $value < $warn->[0])
@@ -458,19 +506,15 @@ sub process_service {
                         . ") exceeded"
                 ));
 
-            if (   !defined $onfield
-                or !defined $onfield->{"state"}
-                or $onfield->{"state"} ne "warning") {
-
+            if ( $oldstate ne "warning") {
                 $hash->{'state_changed'} = 1;
-
             }
         }
-        elsif (defined $onfield and defined $onfield->{"state"} or $force) {
+        else {
             munin_set_var_loc(\%notes, [@$fpath, "state"], "ok");
             munin_set_var_loc(\%notes, [@$fpath, "ok"],    "OK");
 
-	    if ($onfield->{'state'} ne 'ok' or $force) {
+	    if ($oldstate ne 'ok') {
 		$hash->{'state_changed'} = 1;
 	    }
         }
@@ -598,10 +642,26 @@ sub generate_service_message {
             next;
         }
         my $obsess = 0;
-        my $cas = munin_get($contactobj, "always_send");
-        if (defined $cas) {
-            $obsess = grep {scalar(@{$stats{$_}})} (split(/\s+/, lc $cas));
+        my $always_send;
+
+        if (@always_send) {
+            # List of severities from command line argument
+            $always_send = \@always_send;
         }
+        else {
+            # List of severities from contact configuration
+            my $always_send_config = munin_get( $contactobj, "always_send" );
+            my @always_send_config = ($always_send_config);
+            $always_send = \@always_send_config;
+        }
+
+        $always_send = validate_severities($always_send);
+
+        foreach my $cas ( @{$always_send} ) {
+	    if(defined($stats{$cas})) {
+		$obsess += scalar @{$stats{$cas}};
+	    }
+	}
         if (!$hash->{'state_changed'} and !$obsess) {
             next;    # No need to send notification
         }
@@ -756,4 +816,49 @@ sub message_expand {
     return join('', @res);
 }
 
+=pod
+
+Get a list of severities, and return a sorted, lower cased, unique and
+validated list of severities.
+
+Expects and returns an array reference.
+
+If none of the severities given are on the allowed_severities list, it
+will return a reference to an empty array.
+
+=cut
+
+sub validate_severities {
+    my $severities_ref = shift;
+    my @severities     = @{$severities_ref};
+
+    my @allowed_severities = qw{ok warning critical unknown};
+
+    # Flatten, split on comma and whitespace, and lowercase
+    my @expanded_severities =
+      split( /[[:space:],]+/, lc( join( ',', @severities ) ) );
+
+    # Remove duplicates
+    my %seen;
+    my @unique_severities = grep { !$seen{$_}++ } @expanded_severities;
+
+    # Filter on allowed values
+    my %count;
+    my @validated_severities;
+    foreach my $severity ( @unique_severities, @allowed_severities ) {
+        $count{$severity}++;
+    }
+
+    foreach my $severity ( keys %count ) {
+        if ( $count{$severity} == 2 ) {
+            push @validated_severities, $severity;
+        }
+    }
+
+    # Sort the final list
+    my @sorted_severities = sort(@validated_severities);
+
+    return \@sorted_severities;
+}
+
 1;
